const oracledb = require('oracledb');
const OracleConfig = require('./OracleConfig');
const DataBaseLogger = require('../../logs/DataBaseLogger');
const SystemLogger = require('../../logs/SystemLogger');

class Connection {
  static conn = null;

  async connect({ useNewConnection = false } = {}) {
    const hasActiveConnection = await this.hasActiveConnection();
    if(!useNewConnection && hasActiveConnection) {
      return Connection.conn;
    }
    const connParams = OracleConfig.getConnectionParams();
    const newConnection = await oracledb.getConnection(connParams);
    if(!useNewConnection && !hasActiveConnection) {
      Connection.conn = newConnection;
    }
    return newConnection;
  }

  async disconnect() {
    await Connection.conn.close((err) => {
      if (err) {
        SystemLogger.write(err);
      }
    });
  }

  async commit() {
    if(await this.hasActiveConnection()) {
      await Connection.conn.commit();
    }
  }

  async rollback() {
    if (await this.hasActiveConnection()) {
      await Connection.conn.rollback();
    }
  }

  async hasActiveConnection() {
    try {
      if(!Connection.conn) return false;
      if (Object.keys(Connection.conn).length === 0) {
        return false;
      }

      if (typeof Connection.conn.ping !== 'function') {
        return false;
      }
      const err = await Connection.conn.ping();
      if(err) {
        await this.disconnect();
        return false;
      }
      return true;
    } catch(err) {
      return false;
    }
  }

  async getActiveConnection() {
    try {
      if (await this.hasActiveConnection()) {
        return Connection.conn;
      }

      Connection.conn = await this.connect();
      Connection.conn.execute("ALTER SESSION SET TIME_ZONE='UTC'");

      return Connection.conn;
    } catch (error) {
      DataBaseLogger.write(error, {type: 'CONNECTION'});
      return Promise.reject(new Error('DB_ERROR'));
    }
  }

}

module.exports = new Connection();