class UpdaterWarning {
  static state = {}
  static validations = ['INVALID_VERSION', 'DUPLICATE'];
  static defaultState = UpdaterWarning.validations.reduce((option, validationName) => {
    option.DATA = option.DATA || {},
    option[validationName] = false;
    option.DATA[validationName] = {}
    return option;
  }, {});

  getUniqueKey({ unit, casher }) {
    const key = casher ? `${unit}_${casher}` : unit;
    return key;
  }

  getState({ unit, casher = null }) {
    const key = this.getUniqueKey({ unit, casher });
    return UpdaterWarning.state[key];
  }

  getAllState(){
    return UpdaterWarning.state;
  }

  createState({ unit, casher = null}) {
    const key = this.getUniqueKey({ unit, casher });
    if (!UpdaterWarning.state[key]) {
      UpdaterWarning.state[key] = { ...UpdaterWarning.defaultState };
    }
    return UpdaterWarning.state[key];
  }

  addWarning({ unit, casher = null }, validationName, data = {}) {
    const state = this.createState({ unit, casher });
    state[validationName] = true;
    state.DATA[validationName] = { ...state.DATA[validationName], ...data };
  }

  removeWarning({ unit, casher = null }, validationName) {
    const state = this.getState({ unit, casher });
    if (state) {
      state[validationName] = false;
      state.DATA[validationName] = {};
    }
  }

  clearWarning({ unit, casher = null }) {
    const key = this.getUniqueKey({ unit, casher });
    delete UpdaterWarning.state[key];
  }

  hasWarning({ unit, casher = null }, validationName) {
    const state = this.getState(unit, casher);
    return state[validationName];
  }

  getWarning({ unit, casher = null }) {
    return this.getState({ unit, casher });
  }

  getActiveWarnings({ unit, casher = null }) {
    const state = this.getState({ unit, casher });
    if (state) {
      const {
        DATA,
        ...warnings
      } = state;
      return Object.keys(warnings).filter(name => warnings[name]);
    }
    return false;
  }

  getMappedUnitsCashersWithWarning(onlyUnits = false) {
    const unitsWithoutCasher = [];
    const result = Object.keys(UpdaterWarning.state).reduce((unitCasherMap, key) => {
      const [ unit, casher = null ] = key.split('_');
      unitCasherMap[unit] = unitCasherMap[unit] || [];
      if (casher) unitCasherMap[unit].push(casher);
      else unitsWithoutCasher.push(unit);
    }, {});

    return onlyUnits ? unitsWithoutCasher : result;
  }

}

module.exports = new UpdaterWarning();