const fs = require('fs');
const path = require('path');
const ms = require('ms');
const moment = require('moment');

const DateHandler = require('../functions/DateHandler');
const Logger = require('./Logger');

class UpdaterLogger extends Logger {

  constructor() {
    super({
      filePath: process.env.UPDATER_LOG_FOLDER,
      fileExtension: 'zip'
    });
    this.hideConsoleLogger = true;
    this.logFolder = process.env.LOG_FOLDER;
  }

  format(zipContent) {
    return zipContent;
  }

  async writeZip(unit, casher, product, zipContent) {
    const orgName = process.argv[2];
    this.fileName = `logs_org_${orgName}_${product}_filial_${unit}`;
    
    if (casher) {
      this.fileName = `${this.fileName}_cx_${casher}`;
    }
    
    if (!fs.existsSync(this.filePath)) {
      fs.mkdirSync(this.filePath, { recursive: true });
    }

    let response = {};

    try {
      const { fileName } = await this.write(zipContent, {}, { useWriteFile: true });
      response = {
        error: false,
        message: null,
        fileName
      }
    } catch (error) {
      response = { 
        error: true, 
        message: error.message, 
        fileName: null
      }
    }
    return response;
  }

  async getLocalLogInfo(unit, casher, product) {
    return new Promise((resolve, reject) => {
      const sysdate = DateHandler.toString();
      const timeKeepLog = ms(process.env.TIME_KEEP_UPDATER_LOG);
      const orgName = process.argv[2];

      let fileName = `logs_org_${orgName}_${product}_filial_${unit}`;

      if (casher) {
        fileName = `${fileName}_cx_${casher}`;
      }

      fileName = `${fileName}_${sysdate}.zip`;

      fs.stat(path.join(this.filePath, fileName), (err, stats) => {
        let modifiedTime = err ? null : stats.mtime.getTime();
        let lapsedTime = Date.now() - (modifiedTime || 0);

        const useLocalLog = !err && lapsedTime < timeKeepLog;

        resolve({
          useLocalLog,
          fileName
        });
      });
    });
  }

  isOldLog(logDate) {
    return moment(logDate, 'YYYY-MM-DD') < moment(moment().format('YYYY-MM-DD'));
  }

}

module.exports = new UpdaterLogger();