const Environment = require('../helpers/Environment');
const UpdaterLogger = require('../logs/UpdaterLogger');
const DataBaseLogger = require('../logs/DataBaseLogger');
const Connection = require('../database/oracle/Connection');
const SocketManager = require('../socket/SocketManager');
const SystemLogger = require('../logs/SystemLogger');
const Crypt = require('../service/Crypt');
const DB = require('../database/DB');
const SocketEvents = require('../socket/events/SocketEventManager');
const Monitor = require('../socket/events/Monitor');

class StartupTasks {
  runLogTasks() {
    const errorHandler = (err, logType) => SystemLogger.write(err, { type: `Ocorreram erros na limpeza de logs do ${logType}.`});
    const allPromises = [
      UpdaterLogger.startCleanupTask((err) => errorHandler(err, 'Updater')),
      DataBaseLogger.startCleanupTask((err) => errorHandler(err, 'Banco de Dados')),
      SystemLogger.startCleanupTask((err) => errorHandler(err, 'Sistema')),
    ];
    return Promise.allSettled(allPromises);
  }
  async runDbTasks() {
    const { DB_ENCRYPTED, DB_PASSWORD, DB_CRYPT_KEY } = process.env;
    if(DB_ENCRYPTED === 'YES') {
      const dbPassword = Crypt.decrypt(DB_PASSWORD, DB_CRYPT_KEY);
      process.env.DB_PASSWORD = dbPassword;
    } else {
      const encryptedPassword = Crypt.encrypt(DB_PASSWORD, DB_CRYPT_KEY);
      Environment.readFile().then((fileContent) => {
        const lines = fileContent.split('\n');
        const hasEncryptedMnemonic = fileContent.includes('DB_ENCRYPTED');
        const newFileContent = lines.map(line => {
          if(line.includes('DB_PASSWORD')) {
            const passwordMnemonic = `DB_PASSWORD=${encryptedPassword}`;
            if (hasEncryptedMnemonic) {
              return passwordMnemonic;
            } else {
              return `${passwordMnemonic}\nDB_ENCRYPTED=YES`;
            }
          }
          if(line.includes('DB_ENCRYPTED')) {
            return `DB_ENCRYPTED=YES`;
          }
          return line;
        }).join('\n');
        Environment.writeFile(newFileContent).catch(err => {
          SystemLogger.write(err, { type: 'Ocorreu um erro ao atualizar arquivo .env.' });
        });
      }).catch(err => {
        SystemLogger.write(err, { type: 'Ocorreu um erro ao ler arquivo .env.' });
      });
    }
    await Connection.connect();
  }
  async monitoringTasks() {
    await DB.execute(`DELETE FROM ASSISTANT_CONFIG WHERE TIPO = 'I' AND IDMONITORAMENTO IS NULL`);
    Monitor.loadInitialParams();
  }
  async start(server) {
    await this.runLogTasks();
    await this.runDbTasks();
    await this.monitoringTasks();
    SocketManager.connect(server);
    SocketEvents.register(SocketManager.getSocketIo());
  }
}

module.exports = new StartupTasks();