const DB = require('../database/DB');

class updaterProdutosModel 
{  
  getUpdaterProdutos(isActive = false) {
    const WHERE_CLAUSE = isActive ? "WHERE IDATIVO = 'S'" : '';
    const sql = `
      SELECT 
        ID, 
        NMPRODUCT, 
        IDUTMULTVER
      FROM UPDATERPRODUTOS
      ${WHERE_CLAUSE}
      ORDER BY NMPRODUCT
    `;
    return DB.fetch(sql);
  }

  getUpdaterProdutosPerCasherFilial(id) {
    const sql = `
    SELECT  F.CDFILIAL, F.NMFILIAL, F.CDFILIAL || ' - ' || F.NMFILIAL AS CDFILIAL_NMFILIAL
      FROM FILIAL F
    INNER JOIN UPDATERPRODFILI UPF ON UPF.CDFILIAL = F.CDFILIAL
      WHERE UPF.UPDATERPRODID = :PRODUCT_ID
    ORDER BY F.CDFILIAL    
    `; 
    const params = {
      PRODUCT_ID: id,
    }        
    return DB.fetch(sql, params);
  }

  getUpdaterProdutosById(id) {
    const sql = `
      SELECT ID, NMPRODUCT, IDUTMULTVER
        FROM UPDATERPRODUTOS
       WHERE ID = :ID
    `;

    const params = {
      ID: id,
    }

    return DB.fetch(sql, params);
  }

  insertUpdaterProdutos(params) {
    const sql = `
      INSERT INTO UPDATERPRODUTOS
                ( ID,  NMPRODUCT,  IDUTMULTVER, DTINCLUSAO, DTULTATU)
         VALUES (:ID, :NMPRODUCT, :IDUTMULTVER, SYSDATE,    SYSDATE)
    `;
    return DB.execute(sql, params);
  }

  updateUpdaterProdutos(params) {
    const sql = `
      UPDATE UPDATERPRODUTOS
         SET NMPRODUCT   = :NMPRODUCT,
             IDUTMULTVER = :IDUTMULTVER,
             IDATIVO     = 'S',
             DTULTATU    = SYSDATE
       WHERE ID = :ID
    `;
    return DB.execute(sql, params);
  }

  inactiveUpdaterProdutos(id) {
    const sql = `
      UPDATE UPDATERPRODUTOS
         SET IDATIVO  = 'N',
             DTULTATU = SYSDATE
       WHERE ID = :ID
    `;

    const params = {
      ID: id
    }

    return DB.execute(sql, params);
  }

  async getUpdaterProductsWithVersion(P_CDPRODUCTID) {
    const sql = `
    SELECT DISTINCT
      P.ID,
      P.NMPRODUCT,
      F.CDFILIAL,
      F.NMFILIAL,
      E.SGESTADO,
      EV.IDSTEVENTO,
      LISTAGG(NVL(PF.NRVERSAOATUAL, '-'), ';') WITHIN GROUP( ORDER BY PF.ID ) OVER(PARTITION BY PF.CDFILIAL) AS VERSOES,
      LISTAGG(PF.ID, ';') WITHIN GROUP( ORDER BY PF.ID ) OVER(PARTITION BY PF.CDFILIAL) AS PRODFILIIDS
    FROM
      UPDATERPRODFILI PF
      INNER JOIN UPDATERPRODUTOS P ON ( PF.UPDATERPRODID = P.ID )
      INNER JOIN FILIAL          F ON ( PF.CDFILIAL = F.CDFILIAL )
      LEFT JOIN ENDEFILI         E ON ( F.CDFILIAL = E.CDFILIAL )
                                  AND ( E.IDTPENDEFILI = 'P' )
      LEFT JOIN UPDATEREVENTOS  EV ON ( P.ID = EV.CDPRODUCTID )
                                    AND ( PF.ID = EV.CDPRODFILIID )
                                    AND ( PF.CDFILIAL = EV.CDFILIAL )
                                    AND ( EV.IDSTEVENTO NOT IN ( 'FINALIZADO', 'CANCELADO', 'INSTALACAO_PENDENTE' ) )
    WHERE
      P.ID = :P_CDPRODUCTID
    ORDER BY
      F.CDFILIAL
    `;
    const params = { P_CDPRODUCTID };
    return DB.fetch(sql, params);
  }

  async getUpdaterProductsWithVersionAndCasher(P_CDPRODUCTID) {
    const sql = `
    SELECT DISTINCT
      P.ID,
      P.NMPRODUCT,
      F.CDFILIAL,
      F.NMFILIAL,
      CX.CDCAIXA,
      CX.NMCAIXA,
      E.SGESTADO,
      LISTAGG(NVL(PFX.NRVERSAOATUAL, '-'),
              ';') WITHIN GROUP(
      ORDER BY
          PFX.ID
      )
      OVER(PARTITION BY PFX.CDFILIAL,
                        PFX.CDCAIXA)                         AS VERSOES,
      LISTAGG(PFX.ID, ';') WITHIN GROUP(
      ORDER BY
          PFX.ID
      )
      OVER(PARTITION BY PFX.CDFILIAL, PFX.CDCAIXA) AS PRODFILIIDS,
      EVX.IDSTEVENTO
    FROM
      UPDATERPRODFILICX PFX
      INNER JOIN UPDATERPRODUTOS  P  ON ( PFX.UPDATERPRODID = P.ID )
      INNER JOIN FILIAL           F  ON ( PFX.CDFILIAL = F.CDFILIAL )
      INNER JOIN CAIXA            CX ON ( PFX.CDFILIAL = CX.CDFILIAL ) AND
                                        ( PFX.CDCAIXA = CX.CDCAIXA )
      LEFT JOIN ENDEFILI          E  ON ( F.CDFILIAL = E.CDFILIAL ) AND
                                        ( E.IDTPENDEFILI = 'P' )
      LEFT JOIN UPDATEREVENTOSCX EVX ON ( P.ID = EVX.CDPRODUCTID ) AND
                                        ( PFX.ID = EVX.CDPRODFILIID ) AND
                                        ( PFX.CDFILIAL = EVX.CDFILIAL ) AND
                                        ( PFX.CDCAIXA = EVX.CDCAIXA ) AND
                                        ( EVX.IDSTEVENTO NOT IN ( 'FINALIZADO', 'CANCELADO', 'INSTALACAO_PENDENTE' ) )
    WHERE
      P.ID = :P_CDPRODUCTID
    ORDER BY
      CX.CDCAIXA
    `;
    const params = { P_CDPRODUCTID };
    return DB.fetch(sql, params);
  }
}

module.exports = new updaterProdutosModel();
