const axios = require('axios');
const SystemLogger = require('../logs/SystemLogger');
const Gzip = require('../functions/Gzip');
const Base64 = require('../functions/Base64');

class AssistantControl {
  removeStartEndBar(text) {
    return text.replace(/^\/|\/+$/g, "");
  }
  getControlUrl() {
    return this.removeStartEndBar(process.env.ASSISTANT_CONTROL_URL || "");
  }
  getBasicAuth() {
    const { username, password } = this.getCredentials();
    return Base64.encode(`${username}:${password}`);
  }
  getCredentials() {
    const {
      ASSISTANT_CONTROL_USERNAME: username = "",
      ASSISTANT_CONTROL_PASSWORD: password = "",
    } = process.env;
    return { username, password };
  }
  async getProductsConfig() {
    try {
      const url = `${this.getControlUrl()}/product-setting`;
      const response = await axios.get(url, {
        auth: this.getCredentials(),
      });
      return response.data;
    } catch (error) {
      SystemLogger.write(error);
      return null;
    }
  }
  async getProductSettings(productId) {
    try {
      const url = `${this.getControlUrl()}/product-setting/${productId}`;
      const response = await axios.get(url, {
        auth: this.getCredentials(),
      });

      if (response.data.length) {
        const [{ config }] = response.data;
        return Gzip.decode(config, true);
      }

      return null;
    } catch (error) {
      SystemLogger.write(error);
      return null;
    }
  }
  async getProductInfo(productId = "") {
    try {
      const url = `${this.getControlUrl()}/product/${productId}`;
      const response = await axios.get(url, {
        auth: this.getCredentials(),
      });
      return response.data;
    } catch (error) {
      SystemLogger.write(error.message, { type: 'FAIL_GET_PRODUCT_INFO'});
      return null;
    }
  }
  getUpdateAvailableVersion() {
    const version = process.env.UPDATE_AVAILABLE_VERSION;
    return version === "error" ? false : version;
  }

  async getAssistantMonitoring() {
    try {
      const url = `${this.getControlUrl()}/assistant-monitoring`;
      const response = await axios.get(url, {
        auth: this.getCredentials(),
      });
      return response.data;
    } catch (error) {
      SystemLogger.error(error);
      return null;
    }
  }
}

module.exports = new AssistantControl();