const Updater = require("./events/Updater");
const SystemLogger = require("../logs/SystemLogger");
const ms = require("ms");

class ServiceInterval {
  static intervals = {};

  getUniqueKey(CDFILIAL, CDCAIXA = null) {
    const key = CDCAIXA ? `${CDFILIAL}_${CDCAIXA}` : CDFILIAL;
    return key;
  }

  createInterval(event, { CDFILIAL, SERVICES, CDCAIXA = null, FILTERALL = false }) {
    const ID = this.getUniqueKey(CDFILIAL, CDCAIXA);

    if (ID in ServiceInterval.intervals) {
      return;
    }

    ServiceInterval.intervals[ID] = setInterval(async () => {
      try {
        if (FILTERALL) {
          await Updater.emitAll(event, SERVICES, { unit: CDFILIAL, casher: CDCAIXA });
        } else {
          await Updater.emit(event, SERVICES, { unit: CDFILIAL, casher: CDCAIXA });
        }
      } catch(err) {
        SystemLogger.error(err);
      }
    }, ms("10m"));

  }

  clearInterval(CDFILIAL, CDCAIXA = null) {
    const ID = this.getUniqueKey(CDFILIAL, CDCAIXA);

    if (ID in ServiceInterval.intervals) {
      clearInterval(ServiceInterval.intervals[ID]);
      delete ServiceInterval.intervals[ID];
    }
  }
}
module.exports = new ServiceInterval();
