const SocketRooms = require('../SocketRooms');
const SocketManager = require('../SocketManager');

class PortalEmitter {
  rooms = { ...SocketRooms.master };

  getRoomsByEvent(eventName, data) {
    const {
      updatesPage,
      updatesPerCasherPage,
      parametrizationsPage,
      parametrizationsPerCasherPage,
      serviceManagement,
      serviceManagementPerCasher,
    } = this.rooms;

    const useCasher = !!data.CDCAIXA || !!data.P_CDCAIXA;

    switch(eventName) {
      case 'updaterConnected':
        if (useCasher) {
          return [updatesPerCasherPage, parametrizationsPerCasherPage, serviceManagementPerCasher];
        }
        return [updatesPage, parametrizationsPage, serviceManagement];

      case 'updaterWarning':
      case 'selfUpdateSuccess':
      case 'selfUpdateError':
        if (useCasher) {
          return [updatesPerCasherPage, serviceManagementPerCasher];
        }
        return [updatesPage, serviceManagement];

      case 'serviceManagementResult':
        return useCasher ? serviceManagementPerCasher : serviceManagement;
    }
  }

  emit(eventName, data) {
    const roomName = this.getRoomsByEvent(eventName, data);
    SocketManager.getRoom(roomName).emit(eventName, data);
  }
}

module.exports = new PortalEmitter();