const Updater = require('./Updater');
const Monitor = require('./Monitor');
const Portal = require('./Portal');
const SystemLogger = require('../../logs/SystemLogger');
const SocketManager = require('../SocketManager');

class SocketEventManager {
  register(socketIo) {
    socketIo.on("connection", (socket) => {
      socket.on('wsConnectionTest', () => {
          socket.emit('wsConnection', {});
      });
      try {
          const {
            agent,
            origin,
            ...socketInfo
          } = SocketManager.getSocketHeaderInfo(socket);

          const { page } = SocketManager.getQueryParams(socket);
          switch (agent) {
              case "updater":
                  Updater.register(socket, socketInfo);
                  break;
              case "monitor":
                  Monitor.register(socket, socketInfo);
                  break;
              default:
                  Portal.register(socket, { page, origin });
          }

          socket.on("disconnect", async () => {
              const { agent, ...socketInfo } = SocketManager.getSocketHeaderInfo(socket);
              switch(agent.toLowerCase()) {
                  case 'updater':
                      Updater.unregister(socket, socketInfo);
                      break;

                  case 'monitor':
                      Monitor.unregister(socket, socketInfo);
                      break;

                  default: // assistant portal
                      Monitor.disconnectedUser();
              }
          });
        //   UpdaterSocket.registerEvents(socket, agent);
      } catch (error) {
          socket.disconnect();
          SystemLogger.write(error);
      }
  });
  }
}

module.exports = new SocketEventManager();