const SocketRooms = require("../../SocketRooms");
const AssistantConfig = require("../../../model/AssistantConfig");
const Monitor = require("../Monitor");
const SocketManager = require("../../SocketManager");

class MonitoringCloud {
  defaultRoom = SocketRooms.master.monitoringCloudPage;
  static monitoringParams = {};
  static monitoringResult = {};
  static monitoringIntervals = {};

  register(socket) {
    socket.join(this.defaultRoom);
    socket.on("detailMonitoringApache", ({ ID }) => {
      socket.emit(
        "detailMonitoringApacheResult",
        MonitoringCloud.monitoringResult[ID]
      );
    });
    this.startMonitoring();
  }

  hasActiveMonitoring() {
    return Object.keys(MonitoringCloud.monitoringIntervals).length > 0;
  }

  getMonitoringResults() {
    return MonitoringCloud.monitoringResult;
  }

  async getConsummers() {
    const consummers = await this.getRoom(this.defaultRoom).fetchSockets();
    return consummers;
  }

  async hasConsummers() {
    const consummers = await this.getConsummers();
    return consummers.length > 0;
  }

  async loadMonitoringParams() {
    const { rows } = await AssistantConfig.getMonitoringParamsByType({
      TIPO: "C",
    });
    if (!rows.length) return;
    MonitoringCloud.monitoringParams = rows.reduce((allParams, row) => {
      row.COMANDO = Gzip.decode(row.COMANDO);
      allParams[row.ID] = row;
      return allParams;
    }, {});
  }

  updateMonitoringParams(config) {
    const { ID, NOME, DESCRICAO, ATIVO } = config;
    const room = SocketManager.getRoom(this.defaultRoom);
    const parsedId = parseInt(ID, 10);
    if (ATIVO === "S") {
      room.emit("startMonitoring", { ID: parsedId, NOME, DESCRICAO, ATIVO });
      this.startMonitoring(config);
    } else {
      room.emit("stopMonitoring", [parsedId]);
      this.stopMonitoring(parsedId);
    }
  }

  stopMonitoring(ID) {
    const idsToStop = ID
      ? [ID]
      : Object.keys(MonitoringCloud.monitoringIntervals);
    idsToStop.forEach((id) => {
      clearInterval(MonitoringCloud.monitoringIntervals[id]);
    });
  }

  async handleHttpRequest(config) {
    const { ID, COMANDO } = config;
    const response = await Monitor.execHttpRequest(ID, COMANDO.split(";"));
    const failResults = [];
    response.forEach((result) => {
      const { ERROR, STATUS } = result;
      if (STATUS === "N") {
        failResults.push(ERROR);
      }
    });

    MonitoringCloud.monitoringResult[ID] = response;
    const socketResponse = { ID };
    socketResponse.STATUS = failResults.length ? "N" : "S";
    SocketManager.getRoom(this.defaultRoom).emit(
      "apacheCloudResult",
      socketResponse
    );
  }

  createIntervalMonitoring(config) {
    const { ID, INTERVALO } = config;
    const intervalTimeMs = parseInt(INTERVALO, 10) * 1000;
    if (ID in MonitoringCloud.monitoringIntervals) {
      this.stopMonitoring(ID);
    }
    MonitoringCloud.monitoringIntervals[ID] = setInterval(() => {
      this.handleHttpRequest(config);
    }, intervalTimeMs);
  }

  async startMonitoring(config = null) {
    const hasActiveMonitoring = this.hasActiveMonitoring();
    const needCreateOrUpdateMonitoring = config || !hasActiveMonitoring;

    if (!needCreateOrUpdateMonitoring) return;
    const params = config
      ? [config]
      : Object.values(MonitoringCloud.monitoringParams);
    params.forEach((config) => {
      this.handleHttpRequest(config);
      this.createIntervalMonitoring(config);
    });
  }
}

module.exports = new MonitoringCloud();
